#include <stdio.h>
#include <math.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>

#include "ledblur.h"
#include "gp32_functions.h"

SDL_Surface *screen;
SDL_Joystick *joystick;

unsigned short *GpScreen[2];

int scale;
unsigned short gp_palette[256];

unsigned int keys[512];
extern int quit;

int vfirst = 0;
int cur_bpp;
int fullscreen;

void InitSDL()
{
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_AUDIO) < 0)
    	exit2menu();

	screen = SDL_SetVideoMode(320, 240, 16, SDL_HWSURFACE);
	if (!screen)
        exit2menu();

	joystick = SDL_JoystickOpen(0);
	if (!joystick)
        exit2menu();

	SDL_ShowCursor(SDL_DISABLE);
}

int GpGraphicModeSet(int gd_bpp, unsigned short * gp_pal)
{
    if (vfirst!=0)
    {
        free(GpScreen[0]);
        free(GpScreen[1]);
    }

    GpScreen[0] = (unsigned short*)malloc(sizeof(GpScreen[0]) * (LCD_WIDTH * LCD_HEIGHT));
    GpScreen[1] = (unsigned short*)malloc(sizeof(GpScreen[1]) * (LCD_WIDTH * LCD_HEIGHT));
    cur_bpp = gd_bpp;
    vfirst = 1;
}

int GpLcdSurfaceGet(GPDRAWSURFACE * ptgpds, int idx)
{
    ptgpds->bpp = cur_bpp;
    ptgpds->buf_w = LCD_WIDTH;
    ptgpds->buf_h = LCD_HEIGHT;
    ptgpds->ox = 0;
    ptgpds->oy = 0;
    ptgpds->ptbuffer = GpScreen[idx];
}

/*
void GP32toPC(unsigned short *gp32v16, int bpp, SDL_Surface *screen)
{
    int x, y, c;
    unsigned short r, g, b;
    unsigned short *vram;
    vram=(unsigned short*)screen->pixels;

    if (bpp==16)
    {
        unsigned short *gp32vram = (unsigned short*)gp32v16;
        for (x=0; x<LCD_WIDTH; x++)
        {
            int yp = (LCD_HEIGHT - 1) * LCD_WIDTH;
            for (y=LCD_HEIGHT - 1; y>=0; y--)
            {
                c = *gp32vram++;
                r = ((c>>11) & 31)<<11;
                g = ((c>>6) & 31)<<6;
                b = (c>>1) & 31;
                *(vram + yp + x) = r | g | b;
                yp-=LCD_WIDTH;
            }
        }
    }
    else
    {
        unsigned char *gp32vram = (unsigned char*)gp32v16;
        for (x=0; x<LCD_WIDTH; x++)
        {
            int yp = (LCD_HEIGHT - 1) * LCD_WIDTH;
            for (y=LCD_HEIGHT - 1; y>=0; y--)
            {
                *(vram + yp + x) = gp_palette[*gp32vram++];
                yp-=LCD_WIDTH;
            }
        }
    }
}
*/


void GP32toPC(unsigned short *gp32v16, int bpp, SDL_Surface *screen)
{
    int x, y;

    unsigned short *vram = (unsigned short*)screen->pixels;
    unsigned int* vram32 = (unsigned int*)vram;

    if (bpp==16)
    {
        unsigned short *gp32vram = (unsigned short*)gp32v16;
        unsigned int *gp32vram32 = (unsigned int*)gp32vram;
        unsigned int c0, c1;
        for (x=0; x<LCD_WIDTH/2; x++)
        {
            int yp = (LCD_HEIGHT - 1) * (LCD_WIDTH/2) + x;
            for (y=LCD_HEIGHT/2; y!=0; y--)
            {
                c0 = *gp32vram32;
                c0 = (c0 & 0xFFE0FFE0) | ((c0 >>1) & 0x001F001F);

                c1 = *(gp32vram32+LCD_HEIGHT/2);
                c1 = (c1 & 0xFFE0FFE0) | ((c1 >>1) & 0x001F001F);

                *(vram32 + yp) = (c0 & 0xFFFF) | ((c1 & 0xFFFF)<<16);
                *(vram32 + yp - LCD_WIDTH/2) = (c0>>16) | (c1 & 0xFFFF0000);

                gp32vram32++;

                yp-=2*(LCD_WIDTH/2);
            }
            gp32vram32+=(LCD_HEIGHT/2);
        }
    }
    else
    {
        unsigned char *gp32vram = (unsigned char*)gp32v16;
        for (x=0; x<LCD_WIDTH; x++)
        {
            int yp = (LCD_HEIGHT - 1) * LCD_WIDTH;
            for (y=LCD_HEIGHT - 1; y>=0; y--)
            {
                *(vram + yp + x) = gp_palette[*gp32vram++];
                yp-=LCD_WIDTH;
            }
        }
    }
}

void GpSurfaceFlip(GPDRAWSURFACE * ptgpds, int vs)
{
    SDL_LockSurface(screen);
    GP32toPC(ptgpds->ptbuffer, ptgpds->bpp, screen);
    SDL_UnlockSurface(screen);

    SDL_Flip(screen);
}


GP_HPALETTE GpPaletteSelect(GP_HPALETTE h_new)
{
    int i;
    unsigned short r, g, b;
    unsigned short c;
    for (i=0; i<256; i++)
    {
        c = *(h_new + i);
        r = ((c >> 11) & 31)<<11;
        g = ((c >> 6) & 31)<<6;
        b = (c >> 1) & 31;
        gp_palette[i] = r | g | b;
    }    
}

GP_HPALETTE GpPaletteCreate (int entry_num, GP_PALETTEENTRY * pal_entry)
{
    GP_HPALETTE* pal = (GP_HPALETTE*)malloc(sizeof(GP_HPALETTE) * 256);

    int i;
    for (i=0; i<entry_num; i++)
        *(pal+i) = *(pal_entry+i);

    return pal;
}

unsigned int GpPaletteRealize (void){}


unsigned int GpTickCountGet()
{
    return SDL_GetTicks();
}        
